/*
 *  File: TableTimeBarNode.java 
 *  Copyright (c) 2004-2007  Peter Kliem (Peter.Kliem@jaret.de)
 *  A commercial license is available, see http://www.jaret.de.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.jaret.util.ui.timebars.addon.tablesynchronizer;

import de.jaret.util.date.Interval;
import de.jaret.util.ui.table.model.AbstractTableNode;
import de.jaret.util.ui.table.model.ITableNode;
import de.jaret.util.ui.timebars.addon.IHierarchyIdService;
import de.jaret.util.ui.timebars.model.HierarchicalTimeBarModel;
import de.jaret.util.ui.timebars.model.TimeBarNode;
import de.jaret.util.ui.timebars.model.TimeBarNodeListener;
import de.jaret.util.ui.timebars.model.TimeBarRow;
import de.jaret.util.ui.timebars.model.TimeBarRowHeader;

/**
 * TableNode wrapping a TimeBarNode.
 * 
 * @author kliem
 * @version $Id: TableTimeBarNode.java 397 2007-05-01 12:39:10Z olk $
 */
public class TableTimeBarNode extends AbstractTableNode implements TimeBarNodeListener {
    /** wrapped timebar node. */
    protected TimeBarNode _timeBarNode;
    /** id of the node. */
    protected String _id;
    /** hierarchy service is needed for the case that timebarnodes will be added. */
    protected IHierarchyIdService _hierarchyIdService;
    /** every node knows the hierarchical timebar model. */
    protected HierarchicalTimeBarModel _hierarchicalTimeBarModel;

    /**
     * Construct a node. This will also construct all other nodes by descending the tree.
     * 
     * @param idService service providing ids for the timebarnodes
     * @param hierarchicalModel the hierarchical timebar model
     * @param root the root node to begin with
     */
    public TableTimeBarNode(IHierarchyIdService idService, HierarchicalTimeBarModel hierarchicalModel, TimeBarNode root) {
        _hierarchyIdService = idService;
        _hierarchicalTimeBarModel = hierarchicalModel;

        _id = idService.getId(_hierarchicalTimeBarModel, root);
        _timeBarNode = root;
        _timeBarNode.addTimeBarNodeListener(this);
        for (TimeBarNode tbnode : _timeBarNode.getChildren()) {
            addNode(new TableTimeBarNode(_hierarchyIdService, _hierarchicalTimeBarModel, tbnode));
        }
    }

    /**
     * Retrieve the wrapped timebar node.
     * 
     * @return wrapped timebar node
     */
    public TimeBarNode getTimeBarNode() {
        return _timeBarNode;
    }

    /**
     * Find the TableTimeBarNode corresponding to the given timebar node using a recursive descent. This may be
     * optimzed.
     * 
     * @param tbnode timebar node
     * @return corresponding TableTimeBarNode or <code>null</code> if none could be found
     */
    public TableTimeBarNode findChild(TimeBarNode tbnode) {
        if (getTimeBarNode().equals(tbnode)) {
            return this;
        }
        for (ITableNode node : getChildren()) {
            TableTimeBarNode ttbnode = (TableTimeBarNode) node;
            if (ttbnode.getTimeBarNode().equals(tbnode)) {
                return ttbnode;
            }
            // recursive descent
            TableTimeBarNode desc = ttbnode.findChild(tbnode);
            if (desc != null) {
                return desc;
            }
        }
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public String getId() {
        return _id;
    }

    /**
     * {@inheritDoc}
     */
    public void nodeAdded(TimeBarNode parent, TimeBarNode newChild) {
        // TODO Auto-generated method stub

    }

    /**
     * {@inheritDoc}
     */
    public void nodeRemoved(TimeBarNode parent, TimeBarNode removedChild) {
        TableTimeBarNode ttbnode = findChild(removedChild);
        removedChild.removeTimeBarNodeListener(this);
        remNode(ttbnode);
    }

    /**
     * {@inheritDoc}
     */
    public void elementAdded(TimeBarRow row, Interval element) {
        firePropertyChange("elementAdded", null, element);
    }

    /**
     * {@inheritDoc}
     */
    public void elementChanged(TimeBarRow row, Interval element) {
        firePropertyChange("elementChanged", null, element);
    }

    /**
     * {@inheritDoc}
     */
    public void elementRemoved(TimeBarRow row, Interval element) {
        firePropertyChange("elementRemoved", element, null);
    }

    /**
     * {@inheritDoc}
     */
    public void headerChanged(TimeBarRow row, TimeBarRowHeader newHeader) {
        firePropertyChange("headerChanged", null, newHeader);
    }

    /**
     * {@inheritDoc}
     */
    public void rowDataChanged(TimeBarRow row) {
        firePropertyChange("dataChanged", null, row);
    }

}
