/*
 *  File: TableTimeBarSynchronizer.java 
 *  Copyright (c) 2004-2007  Peter Kliem (Peter.Kliem@jaret.de)
 *  A commercial license is available, see http://www.jaret.de.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.jaret.util.ui.timebars.addon.tablesynchronizer;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import de.jaret.util.ui.table.JaretTable;
import de.jaret.util.ui.table.model.IColumn;
import de.jaret.util.ui.table.model.IHierarchicalJaretTableModel;
import de.jaret.util.ui.table.model.IHierarchicalTableViewState;
import de.jaret.util.ui.table.model.IHierarchicalTableViewStateListener;
import de.jaret.util.ui.table.model.IJaretTableCell;
import de.jaret.util.ui.table.model.IJaretTableSelectionModelListener;
import de.jaret.util.ui.table.model.IRow;
import de.jaret.util.ui.table.model.ITableNode;
import de.jaret.util.ui.table.model.ITableViewState;
import de.jaret.util.ui.table.model.ITableViewStateListener;
import de.jaret.util.ui.table.model.ITableViewState.RowHeightMode;
import de.jaret.util.ui.table.renderer.ICellStyle;
import de.jaret.util.ui.timebars.model.HierarchicalViewStateListener;
import de.jaret.util.ui.timebars.model.ITimeBarViewStateListener;
import de.jaret.util.ui.timebars.model.TimeBarNode;
import de.jaret.util.ui.timebars.model.TimeBarRow;
import de.jaret.util.ui.timebars.model.TimeBarSelectionListener;
import de.jaret.util.ui.timebars.model.TimeBarSelectionModel;
import de.jaret.util.ui.timebars.swt.TimeBarViewer;

/**
 * Helper that synchronizes a jaret table with a timebar viewer. There are several requirements that have to be met by
 * the participating components:
 * <ul>
 * <li>the timescale of the tbv has to be in top position; the header of the table should be able to cope with large
 * heights, since it will be used to align the table vertically</li>
 * <li></li>
 * </ul>
 * 
 * @author kliem
 * @version $Id: TableTimeBarSynchronizer.java 536 2007-08-19 10:49:10Z olk $
 */
public class TableTimeBarSynchronizer implements PropertyChangeListener, HierarchicalViewStateListener {
    /** the table. */
    protected JaretTable _table;
    /** the timebar viewer. */
    protected TimeBarViewer _tbv;

    /**
     * Construct the synchronizer for a table and a timebar viewer.
     * 
     * @param table jaret table
     * @param tbv timebar viewer
     */
    public TableTimeBarSynchronizer(JaretTable table, TimeBarViewer tbv) {
        _table = table;
        _tbv = tbv;

        // set up the table
        if (_tbv.getTimeBarViewState().getUseVariableRowHeights()) {
            _table.getTableViewState().setRowHeightMode(ITableViewState.RowHeightMode.VARIABLE);
        } else {
            _table.getTableViewState().setRowHeightMode(ITableViewState.RowHeightMode.FIXED);
        }
        _table.getTableViewState().setRowHeight(_tbv.getRowHeight());
        // initial adjustment of the table header
        adjustTableHeader();
        // the synchronizer is propChangeLIstener for both widgets
        _tbv.addPropertyChangeListener(this);
        _table.addPropertyChangeListener(this);

        // setup row height synchronization
        registerRowHeightSynchronization();

        // setup selection synchronization
        registerSelectionSynchronization();

        // set up hierarchy synchronization
        if (_tbv.getHierarchicalModel() != null) {
            // hierarchical models
            // sorting the table does not make any sense in a hierarchical world
            _table.setAllowSorting(false);

            _tbv.getHierarchicalViewState().addHierarchicalViewstateListener(this);
            ((IHierarchicalTableViewState) _table.getTableViewState())
                    .addHierarchicalViewstateListener(new IHierarchicalTableViewStateListener() {

                        /**
                         * {inheritDoc} listening to the table.
                         */
                        public void nodeExpanded(ITableNode node) {
                            TimeBarNode tbnode = ((TableTimeBarNode) node).getTimeBarNode();
                            _tbv.getHierarchicalViewState().setExpanded(tbnode, true);
                        }

                        /**
                         * {inheritDoc} listening to the table.
                         */
                        public void nodeFolded(ITableNode node) {
                            TimeBarNode tbnode = ((TableTimeBarNode) node).getTimeBarNode();
                            _tbv.getHierarchicalViewState().setExpanded(tbnode, false);
                        }

                    });
        } else {
            // if not hierachical model set up sorter and filter
            TimeBarTableRowSorter tbtrs = new TimeBarTableRowSorter((TableTimeBarModel) _table.getTableModel(), _table);
            _tbv.setRowSorter(tbtrs);
            TimeBarTableRowFilter tbtrf = new TimeBarTableRowFilter((TableTimeBarModel) _table.getTableModel(), _table);
            _tbv.setRowFilter(tbtrf);
        }

    }

    /**
     * Setup listeners for synchronizig changes in the row heights.
     */
    private void registerRowHeightSynchronization() {
        _tbv.getTimeBarViewState().addTimeBarViewStateListener(new ITimeBarViewStateListener() {

            public void rowHeightChanged(TimeBarRow row, int newHeight) {
                TableTimeBarRow ttbr = ((TableTimeBarModel) _table.getTableModel()).getTableTimeBarRow(row);
                _table.getTableViewState().setRowHeight(ttbr, newHeight);
            }

            public void viewStateChanged() {
                // all heights may have changed
                for (int i = 0; i < _tbv.getModel().getRowCount(); i++) {
                    TimeBarRow row = _tbv.getModel().getRow(i);
                    TableTimeBarRow ttbr = ((TableTimeBarModel) _table.getTableModel()).getTableTimeBarRow(row);
                    _table.getTableViewState().setRowHeight(ttbr, _tbv.getTimeBarViewState().getRowHeight(row));
                }
            }

        });

        _table.getTableViewState().addTableViewStateListener(new ITableViewStateListener() {

            public void cellStyleChanged(IRow row, IColumn column, ICellStyle cellStyle) {
            }

            public void columnOrderChanged() {
            }

            public void columnVisibilityChanged(IColumn column, boolean visible) {
            }

            public void columnWidthChanged(IColumn column, int newWidth) {
            }

            public void columnWidthsChanged() {
            }

            public void rowHeightChanged(IRow row, int newHeight) {
                TableTimeBarRow ttbr = (TableTimeBarRow) row;
                _tbv.getTimeBarViewState().setRowHeight(ttbr.getTimeBarRow(), newHeight);
            }

            public void rowHeightModeChanged(IRow row, RowHeightMode newHeightMode) {
            }

            public void sortingChanged() {
            }

        });

    }

    /**
     * Sets up listeners for selection on the components. Seletion will be synchronized on row level only, since
     * intervals can not be mapped to cells.
     */
    private void registerSelectionSynchronization() {
        _tbv.getSelectionModel().addTimeBarSelectionListener(new TimeBarSelectionListener() {

            /**
             * {@inheritDoc} Listening to the TBV.
             */
            public void elementAddedToSelection(TimeBarSelectionModel selectionModel, Object element) {
                if (element instanceof TimeBarNode) {
                    TimeBarNode tbnode = (TimeBarNode) element;
                    IHierarchicalJaretTableModel hTableModel = _table.getHierarchicalModel();
                    TableTimeBarNode ttbn = (TableTimeBarNode) hTableModel.getRootNode();
                    _table.getSelectionModel().addSelectedRow(ttbn.findChild(tbnode));
                } else if (element instanceof TimeBarRow) {
                    // flat
                    TimeBarRow tbRow = (TimeBarRow) element;
                    TableTimeBarRow ttbr = ((TableTimeBarModel) _table.getTableModel()).getTableTimeBarRow(tbRow);
                    _table.getSelectionModel().addSelectedRow(ttbr);
                }
            }

            /**
             * {@inheritDoc} Listening to the TBV.
             */
            public void elementRemovedFromSelection(TimeBarSelectionModel selectionModel, Object element) {
                if (element instanceof TimeBarNode) {
                    TimeBarNode tbnode = (TimeBarNode) element;
                    IHierarchicalJaretTableModel hTableModel = _table.getHierarchicalModel();
                    TableTimeBarNode ttbn = (TableTimeBarNode) hTableModel.getRootNode();
                    _table.getSelectionModel().remSelectedRow(ttbn.findChild(tbnode));
                } else if (element instanceof TimeBarRow) {
                    // flat
                    TimeBarRow tbRow = (TimeBarRow) element;
                    TableTimeBarRow ttbr = ((TableTimeBarModel) _table.getTableModel()).getTableTimeBarRow(tbRow);
                    _table.getSelectionModel().remSelectedRow(ttbr);
                }
            }

            /**
             * {@inheritDoc} Listening to the TBV.
             */
            public void selectionChanged(TimeBarSelectionModel selectionModel) {
                _table.getSelectionModel().clearSelection();
                for (TimeBarRow tbRow : selectionModel.getSelectedRows()) {
                    if (tbRow instanceof TimeBarNode) {
                        // hierarchy
                        TimeBarNode tbnode = (TimeBarNode) tbRow;
                        IHierarchicalJaretTableModel hTableModel = _table.getHierarchicalModel();
                        TableTimeBarNode ttbn = (TableTimeBarNode) hTableModel.getRootNode();
                        _table.getSelectionModel().addSelectedRow(ttbn.findChild(tbnode));
                    } else {
                        // flat
                        TableTimeBarRow ttbr = ((TableTimeBarModel) _table.getTableModel()).getTableTimeBarRow(tbRow);
                        _table.getSelectionModel().addSelectedRow(ttbr);
                    }
                }

            }

        });

        _table.getSelectionModel().addTableSelectionModelListener(new IJaretTableSelectionModelListener() {

            public void cellSelectionAdded(IJaretTableCell cell) {
            }

            public void cellSelectionRemoved(IJaretTableCell cell) {
            }

            public void columnSelectionAdded(IColumn column) {
            }

            public void columnSelectionRemoved(IColumn column) {
            }

            public void rowSelectionAdded(IRow row) {
                if (row instanceof TableTimeBarNode) {
                    TableTimeBarNode ttbn = (TableTimeBarNode) row;
                    _tbv.getSelectionModel().addSelectedRow(ttbn.getTimeBarNode());
                } else {
                    // flat
                    TableTimeBarRow ttbr = (TableTimeBarRow) row;
                    _tbv.getSelectionModel().addSelectedRow(ttbr.getTimeBarRow());
                }
            }

            public void rowSelectionRemoved(IRow row) {
                if (row instanceof TableTimeBarNode) {
                    TableTimeBarNode ttbn = (TableTimeBarNode) row;
                    _tbv.getSelectionModel().remSelectedRow(ttbn.getTimeBarNode());
                } else {
                    // flat
                    TableTimeBarRow ttbr = (TableTimeBarRow) row;
                    System.out.println("REM " + ttbr.getTimeBarRow());
                    _tbv.getSelectionModel().remSelectedRow(ttbr.getTimeBarRow());
                }
            }

        });

    }

    /**
     * Adjust the table header according to the table viewer.
     */
    private void adjustTableHeader() {
        int height = _tbv.getXAxisHeight();
        height = height - _table.getAutoFilterHeight();
        _table.setHeaderHeight(height);
    }

    /**
     * {@inheritDoc} handle the property changes and deliver them to the other component.
     */
    public void propertyChange(PropertyChangeEvent evt) {
        if (evt.getSource() == _tbv) {
            if (evt.getPropertyName().equals(TimeBarViewer.PROPERTYNAME_ROWHEIGHT)) {
                _table.getTableViewState().setRowHeight(_tbv.getRowHeight());
            } else if (evt.getPropertyName().equals(TimeBarViewer.PROPERTYNAME_XAXISHEIGHT)) {
                adjustTableHeader();
            } else if (evt.getPropertyName().equals(TimeBarViewer.PROPERTYNAME_FIRSTROW)
                    || evt.getPropertyName().equals(TimeBarViewer.PROPERTYNAME_FIRSTROWOFFSET)) {
                _table.setFirstRow(_tbv.getFirstRowDisplayed(), _tbv.getFirstRowOffset());
            } else if (evt.getPropertyName().equals(TimeBarViewer.PROPERTYNAME_ROWHEIGHTDRAGGINGALLOWED)) {
                _table.setRowResizeAllowed(_tbv.isRowHeightDragginAllowed());
                _table.getTableViewState().setRowHeightMode(ITableViewState.RowHeightMode.VARIABLE);
            }
        } else {
            // table is the source
            if (evt.getPropertyName().equals(JaretTable.PROPERTYNAME_HEADERHEIGHT)) {
                _tbv.setXAxisHeight(_table.getHeaderHeight() + _table.getAutoFilterHeight());
            } else if (evt.getPropertyName().equals(JaretTable.PROPERTYNAME_FIRSTROWPIXELOFFSET)
                    || evt.equals(JaretTable.PROPERTYNAME_FIRSTROWIDX)) {
                _tbv.setFirstRow(_table.getFirstRowIdx(), _table.getFirstRowPixelOffset());
            } else if (evt.getPropertyName().equals(JaretTable.PROPERTYNAME_AUTOFILTERENABLE)) {
                adjustTableHeader();
            }
        }

    }

    /**
     * {@inheritDoc} listening to the TBV.
     */
    public void nodeExpanded(TimeBarNode node) {
        IHierarchicalJaretTableModel hTableModel = _table.getHierarchicalModel();
        TableTimeBarNode ttbn = (TableTimeBarNode) hTableModel.getRootNode();
        ((IHierarchicalTableViewState) _table.getTableViewState()).setExpanded(ttbn.findChild(node), true);
    }

    /**
     * {@inheritDoc} listening to the TBV.
     */
    public void nodeFolded(TimeBarNode node) {
        IHierarchicalJaretTableModel hTableModel = _table.getHierarchicalModel();
        TableTimeBarNode ttbn = (TableTimeBarNode) hTableModel.getRootNode();
        ((IHierarchicalTableViewState) _table.getTableViewState()).setExpanded(ttbn.findChild(node), false);
    }

}
